<?php
namespace Teknisa\CRM\Controller;

use Teknisa\CRM\Controller\TeknisaCrud;
use Teknisa\CRM\Controller\ReplicacaoParceiroController as Replicacao;
use Teknisa\CRM\Service\ParceiroNegocioService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class ParceiroNegocioController extends TeknisaCrud {

    protected $crudService;
    protected $dataSourceName = 'crm_parcnegocio';
    protected $dataSourceName2 = 'statusParcnegocio';
    protected $dataSourceNameGetStatus = 'getStatusParc';
    protected $parameterBagColumns = array('IDTPOPERADOR', 'NRREVENDA', 'IDNOMETAG', 'P_CRM_NRORGTRAB');
    protected $parceiroService;
    protected $connection;
    protected $environment;

    public function __construct($crudService, ParameterBag $parameterBag, ParceiroNegocioService $parceiroService, $tagController, Replicacao $replicacao, EntityManager $entityManager, $environment) {
        $this->crudService = $crudService;
        $this->parceiroService = $parceiroService;
        $this->tagController = $tagController;
        $this->replicacao = $replicacao;
        $this->connection = $entityManager->getConnection();
        $this->environment = $environment;
        parent::__construct($crudService, $parameterBag);
    }
    
    public function find(DTO\Request\Filter $request,DTO\Response $response) {
        $hasTagFilter = false;
        $this->dataSourceName = $this->dataSourceName;
        
        foreach($request->getFilterCriteria()->getConditions() as $filter){
            if($filter['columnName'] === 'IDNOMETAG')
                $hasTagFilter = true;
        }
        if(!$hasTagFilter)
            $request->getFilterCriteria()->addCondition('IDNOMETAG', '=', '');
        parent::find($request, $response);    
    }
    
    public function getParcNegocio(DTO\Request\Filter $request,DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceName2;
        parent::find($request, $response);    
    }
    
    public function deleteContato(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        $this->parceiroService->deleteContato($row);  
    }
    public function findContato(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'crm_relacionaparc';
        $this->parameterBagColumns = array('P_NRORG');
        parent::find($request, $response);
    }
    
    public function getStatusParc(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = "getStatusParc";
        $this->parameterBagColumns = array('P_NRPARCNEGOCIO', 'P_NRORG');
        parent::find($request, $response);
    }
    
    public function findRelacionamento(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'crm_tiporelaciona';
        $this->parameterBagColumns = array();
        parent::find($request, $response);
    }
    
    public function saveContato(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $parcnegocio = $this->parceiroService->saveContato($row);
            $response->addDataset(new DataSet('columnsToReturn', array("NRPARCNEGOCIO" => $parcnegocio['NRPARCNEGOCIO'], "NRRELACIONAPARC" =>$parcnegocio['NRRELACIONAPARC'])));
            
        }
        catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function save(DTO\Request $request, DTO\Response $response) {
        $this->connection->beginTransaction();
        
        try {
            $row = $request->getRow();
            
            $nrparcnegocio = $this->parceiroService->saveParceiro($row);
            
            $row['IDNOMETAG'] = isset($row['IDNOMETAG']) ? $row['IDNOMETAG'] : '';
            $this->tagController->saveOrUpdateTags($nrparcnegocio, $row['IDNOMETAG'], 'PARCNEGOCIO');
            $replicacao = $this->replicacao->replica($nrparcnegocio, $row['CDTIPOPARCPRINCIPAL'], 'PARCEIRO');
            $response->addDataSet(new DataSet("REPLICACAO", array($replicacao)));
            if ($row['__is_new']) {
                $response->addDataset(new DataSet('columnsToReturn', array("NRPARCNEGOCIO" => $nrparcnegocio)));
            }
            
            $this->parceiroService->updateResponsavel($row);
            
            $this->connection->commit();
            $this->parceiroService->updateResponsavel($row);
        }
        catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
            $this->connection->rollBack();
        }
    }
        
    public function procedureMergeParc(DTO\Request\Row $row, DTO\Response $response) {
        $row = $row->getRow();

        try {
            $nrparcorigemArray  = $row['NRPARCORIGEM'];
            $nrparcdestino      = $row['NRPARCDESTINO'];
            $cdoperevol         = $this->environment->getCdOperador();
            $nrorg              = $this->environment->getNrOrgTrab() !== null ?
                $this->environment->getNrOrgTrab() : $this->environment->getNrOrg();
                
            $retorno            = "";
            $retornoerro        = "";
            
            $stringfyParceiros = $this->parceiroService->
                stringfyParceiros($nrparcorigemArray);
            
            foreach($nrparcorigemArray as $nrparcori) {
                $rowToCheck = array(
                    "NRPARCORI"     =>  $nrparcori,
                    "NRPARCDEST"    =>  $nrparcdestino,
                    "NRORG"         =>  $nrorg
                );
                                
                $canMerge = $this->parceiroService->checkParcsCanMerge($rowToCheck);
            }
            
            $procedureFundirParc = new StoredProcedure($this->connection, "FUNDIR_PARCNEGOCIO_CRM");
            $procedureFundirParc->addParam(new Param('P_PARCORIGEM', Param::PARAM_INPUT, $stringfyParceiros));
            $procedureFundirParc->addParam(new Param('P_PARCNEGDESTINO', Param::PARAM_INPUT, $nrparcdestino));
            $procedureFundirParc->addParam(new Param('P_NRORG', Param::PARAM_INPUT, $nrorg));
            $procedureFundirParc->addParam(new Param('P_CDOPERULTATU', Param::PARAM_INPUT, $cdoperevol));
            $procedureFundirParc->addParam(new Param('P_RETORNO', Param::PARAM_OUTPUT, $retorno));
            $procedureFundirParc->addParam(new Param('P_RETORNOERRO', Param::PARAM_OUTPUT, $retornoerro));

 
            $procedureFundirParc->execute();
            $response->addDataSet(new DataSet("DADOS", array("RETORNO" => $retorno, "RETORNOERRO" => $retornoerro)));
        }catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function procedureUnmergeParc(DTO\Request\Row $row, DTO\Response $response) {
        $row = $row->getRow();
        
        try {
            $this->connection->beginTransaction();
            
            
    
            $nrParcsADesfundir      = $row['PARCSDESFUNDIR'];
            $nrparcatual            = $row['NRPARCATUAL'];
            $cdoperevol             = $this->environment->getCdOperador();
            $nrorg                  = $this->environment->getNrOrgTrab();
            
            $desfundidos = array();
            $naoDesfundidos = array();
            $erros = array();
            
            foreach($nrParcsADesfundir as $nrParcDesfusao) {
                $parcNome = "";
                
                try{
                    $paramsDesc = array(
                        ":NRPARCDEST"   =>  $nrParcDesfusao,
                        ":NRORG"        =>  $nrorg
                    );
                    
                    $rowToCheck = array(
                        "NRPARCORI"  => $nrParcDesfusao,
                        "NRPARCDEST" => $nrparcatual,
                        "NRORG"      => $nrorg
                    );
                    
                    $parcNome = $this->parceiroService->findParcName($nrParcDesfusao);
                    
                    $parcsCanUnmerge = $this->parceiroService->checkParcsCanUnmerge($rowToCheck);
                    
                    $return = array(
                        "nome"          =>  $parcNome,
                        "nrparcnegocio" =>  $nrParcDesfusao
                    );
                    
                    $descendentes = array();
                    array_push($descendentes, $nrParcDesfusao);
                    $this->parceiroService->findDescendentesParc($paramsDesc, $descendentes);
                    
                    $strDescendentes = $this->parceiroService->
                        stringfyParceiros($descendentes);
                        
                    $procedureDesfundirParc = new StoredProcedure($this->connection, "DESFUNDIR_PARCNEGOCIO_CRM");
                    
                    $procedureDesfundirParc->addParam(new Param('P_PARCNOVODEST',
                        Param::PARAM_INPUT, $nrParcDesfusao));
                        
                    $procedureDesfundirParc->addParam(new Param('P_PARCATUAL',
                        Param::PARAM_INPUT, $nrparcatual));
                    
                    $procedureDesfundirParc->addParam(new Param('P_STRDESCENDENTES',
                        Param::PARAM_INPUT, $strDescendentes));
                        
                    $procedureDesfundirParc->addParam(new Param('P_NRORG',
                        Param::PARAM_INPUT, $nrorg));
                        
                    $procedureDesfundirParc->addParam(new Param('P_CDOPERADOR',
                        Param::PARAM_INPUT, $cdoperevol));
                
                    $procedureDesfundirParc->execute();
                    $this->parceiroService->desativarFusao($nrParcDesfusao,
                        $nrparcatual, $cdoperevol, $nrorg);
                    
                    array_push($desfundidos, $return);
                }catch (\Exception $e) {
                    $return = array(
                        "nome"          =>  $parcNome,
                        "nrparcnegocio" =>  $nrParcDesfusao
                    );
                    
                    array_push($naoDesfundidos, $return);
                    array_push($erros, "$nrParcDesfusao: {$e->getMessage()}");
                }
            }
            
            $response->addDataSet(new DataSet("DADOS", array(
                    "DESFUNDIDOS"       => $desfundidos,
                    "NAODESFUNDIDOS"    => $naoDesfundidos,
                    "ERROS"             => $erros)));
            
            $this->connection->commit();
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
            $this->connection->rollback();
        }
    }
    
    public function checkParcHasMerge(DTO\Request\Row $row, DTO\Response $response) {
        $row = $row->getRow();
        
        try {
            $hasMerge = $this->parceiroService->checkParcHasMerge($row);
            $response->addDataSet(new DataSet("DADOS", array("HASMERGE" => $hasMerge)));
        } catch(Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
        
    }
    
    public function transfererirOperadores(DTO\Request\Row $request, DTO\Response $response){
        $rows = $request->getRow();

        $this->parceiroService->transfererirOperadores($rows);
    }
}