<?php
namespace Controller;

use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Service\PerfilAcessoRepository;
use ZeedhiUser\Controllers\General\ZhuCrud;
use ZeedhiUser\Services\AccessControl\Environment;

class PerfilAcessoController extends ZhuCrud {

    protected $dataSourceName = "perfilacesso";
    protected $perfilAcessoRepository;

    public function __construct(Manager $dataSourceManager, Environment $environment,
                                ParameterBag $parameterBag, PerfilAcessoRepository $perfilAcessoRepository) {
        $this->perfilAcessoRepository = $perfilAcessoRepository;
        parent::__construct($dataSourceManager, $environment, $parameterBag);
    }

    public function save(DTO\Request\DataSet $request, DTO\Response $response) {
        $index = 1;
        try {
            $maxNRPERFILACESSO = $this->perfilAcessoRepository->getMaxNrPerfilAcesso();
        }
        catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage()));
            $maxNRPERFILACESSO = 1;
        }

        foreach ($request->getDataSet()->getRows() as $row) {
            if ($row->get("__is_new")) {
                $row->set("NRPERFILACESSO", $maxNRPERFILACESSO+$index);
                $index++;
            }

            $firstRow = current($request->getDataSet()->getRows());
            $firstRow->set("NRUSERGROUP", null);
            $firstRow->set("NRPERFACESSOPAI", null);
        }

        parent::save($request, $response);
    }

    public function delete(DTO\Request\DataSet $request, DTO\Response $response) {
        $rows = $request->getDataSet()->getRows();
        $perfisToDelete = array();
        foreach ($rows as $row) {
            $perfisToDelete[] = $row->get("NRPERFILACESSO");
        }
        $params = array (
            "NRORG" =>  $this->environment->getOrganizationId()
        );
        try {
            $this->perfilAcessoRepository->deleteAssocietedRules($perfisToDelete, $params);
            $this->perfilAcessoRepository->deleteAssocietedPerfAcessParc($perfisToDelete, $params);
        }
        catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage()));
        }

        parent::delete($request, $response);
    }
}