/* Sistemas Integrados TECNISA
   Servios

Nome: LIMPA_FINANCEIRO

Descrio:
   Apaga os registros das tabelas do financeiro

Criao:
03/05/2001 - Andr Rodrigo

EXECUTE LIMPA_FINANCEIRO('01', TO_DATE('31/05/2000'));

*/
CREATE OR REPLACE
PROCEDURE LIMPA_FINANCEIRO
   (P_CDEMPRESA BOLETRECEBER.CDEMPRESA%TYPE,
    P_DTLIMITE  DATE) IS

    V_CTFILIAL   NUMBER;
    V_CTBAIXA    NUMBER;
    V_CTMOVFLUXO NUMBER;
    V_CTDEBI     NUMBER;
    V_CTCRED     NUMBER;
    V_CTTRANSF   NUMBER;
    V_CTMOV      NUMBER;
    V_DTBAIXA    DATE;
    V_DTBOLETO   DATE;
    V_CTBOL      NUMBER;
BEGIN

/*  EXCLUSO DAS CONTAS A PAGAR E BOLETOS  */

   SELECT
      COUNT(CDFILIAL)
   INTO
      V_CTFILIAL
   FROM
      BOLETPAGAR
   WHERE
      (CDEMPRESA   = P_CDEMPRESA)   AND
      (DTVENBOLPAG <= P_DTLIMITE);

   IF V_CTFILIAL > 0 THEN
      FOR C_BOLPAG IN
         (SELECT
             CDFILIAL,
             NRLANBOLPAG,
             CDFILIBAIPAG,
             NRBAIXAPAG
          FROM
             BOLETPAGAR
          WHERE
             (CDEMPRESA    = P_CDEMPRESA)  AND
             (DTVENBOLPAG  <= P_DTLIMITE)) LOOP

          IF C_BOLPAG.NRBAIXAPAG IS NOT NULL THEN
             /* VERIFICA SE O BOLETO FOI BAIXADO EM UMA DATA POSTERIOR A DATA LIMITE */
             SELECT
                DTBAIXAPAG
             INTO
                V_DTBAIXA
             FROM BAIXAPAGA
             WHERE
                (CDFILIAL   = C_BOLPAG.CDFILIBAIPAG) AND
                (NRBAIXAPAG = C_BOLPAG.NRBAIXAPAG);

             /* VERIFICA SE O BOLETO PERTENCE A UMA BAIXA QUE POSSUI ALGUM BOLETO COM VENCIMENTO POSTERIOR A DATA DA BAIXA */
             SELECT
                MAX(DTVENBOLPAG)
             INTO
                V_DTBOLETO
             FROM BOLETPAGAR
             WHERE
                (CDFILIBAIPAG = C_BOLPAG.CDFILIBAIPAG) AND
                (NRBAIXAPAG   = C_BOLPAG.NRBAIXAPAG);
          END IF;

          IF (C_BOLPAG.NRBAIXAPAG IS NULL) OR
             ((V_DTBAIXA  <= P_DTLIMITE)   AND
              (V_DTBOLETO <= P_DTLIMITE))  THEN

             FOR C_CNTPAG IN
                (SELECT
                    CDFILIAL,
                    NRLANCTONF,
                    NRSEQVENPAG,
                    NRPARCELAPAG
                 FROM CONTAPAGAR
                 WHERE
                    (CDFILIBPAG   = C_BOLPAG.CDFILIAL)     AND
                    (NRLANBOLPAG  = C_BOLPAG.NRLANBOLPAG)) LOOP

                  DELETE FROM RATCTPAGAR
                  WHERE
                     (CDFILIAL     = C_CNTPAG.CDFILIAL)     AND
                     (NRLANCTONF   = C_CNTPAG.NRLANCTONF)   AND
                     (NRSEQVENPAG  = C_CNTPAG.NRSEQVENPAG)  AND
                     (NRPARCELAPAG = C_CNTPAG.NRPARCELAPAG);

                  DELETE FROM AJCTPAGAR
                  WHERE
                     (CDFILIAL     = C_CNTPAG.CDFILIAL)     AND
                     (NRLANCTONF   = C_CNTPAG.NRLANCTONF)   AND
                     (NRSEQVENPAG  = C_CNTPAG.NRSEQVENPAG)  AND
                     (NRPARCELAPAG = C_CNTPAG.NRPARCELAPAG);

                  DELETE FROM CONTAPAGAR
                  WHERE
                     (CDFILIAL     = C_CNTPAG.CDFILIAL)     AND
                     (NRLANCTONF   = C_CNTPAG.NRLANCTONF)   AND
                     (NRSEQVENPAG  = C_CNTPAG.NRSEQVENPAG)  AND
                     (NRPARCELAPAG = C_CNTPAG.NRPARCELAPAG);
             END LOOP;

             DELETE FROM AJBOLPAG
             WHERE
                (CDFILIAL    = C_BOLPAG.CDFILIAL) AND
                (NRLANBOLPAG = C_BOLPAG.NRLANBOLPAG);

             DELETE FROM ERRCNABBOLP
             WHERE
                (CDFILIAL    = C_BOLPAG.CDFILIAL) AND
                (NRLANBOLPAG = C_BOLPAG.NRLANBOLPAG);

             DELETE FROM BOLETPAGAR
             WHERE
                (CDFILIAL    = C_BOLPAG.CDFILIAL) AND
                (NRLANBOLPAG = C_BOLPAG.NRLANBOLPAG);
          END IF;
      END LOOP;
   END IF;

/*  EXCLUSO DAS CONTAS A RECEBER E BOLETOS  */

   SELECT
      COUNT(CDFILIAL)
   INTO
      V_CTFILIAL
   FROM
      BOLETRECEBER
   WHERE
      (CDEMPRESA   = P_CDEMPRESA)   AND
      (DTVENBOLREC <= P_DTLIMITE);

   IF V_CTFILIAL > 0 THEN
      FOR C_BOLREC IN
         (SELECT
             CDFILIAL,
             NRLANBOLREC,
             CDFILIBAIREC,
             NRBAIXAREC
          FROM
             BOLETRECEBER
          WHERE
             (CDEMPRESA    = P_CDEMPRESA)  AND
             (DTVENBOLREC  <= P_DTLIMITE)) LOOP

          IF C_BOLREC.NRBAIXAREC IS NOT NULL THEN
             /* VERIFICA SE O BOLETO FOI BAIXADO EM UMA DATA POSTERIOR A DATA LIMITE */
             SELECT
                DTBAIXAREC
             INTO
                V_DTBAIXA
             FROM BAIXARECEBIDA
             WHERE
                (CDFILIAL   = C_BOLREC.CDFILIBAIREC) AND
                (NRBAIXAREC = C_BOLREC.NRBAIXAREC);

             /* VERIFICA SE O BOLETO PERTENCE A UMA BAIXA QUE POSSUI ALGUM BOLETO COM VENCIMENTO POSTERIOR A DATA DA BAIXA */
             SELECT
                MAX(DTVENBOLREC)
             INTO
                V_DTBOLETO
             FROM BOLETRECEBER
             WHERE
                (CDFILIBAIREC = C_BOLREC.CDFILIBAIREC) AND
                (NRBAIXAREC   = C_BOLREC.NRBAIXAREC);
          END IF;

          IF (C_BOLREC.NRBAIXAREC IS NULL)  OR
             ((V_DTBAIXA  <= P_DTLIMITE)    AND
              (V_DTBOLETO <= P_DTLIMITE))   THEN

             FOR C_CNTREC IN
                (SELECT
                    CDFILIAL,
                    NRLANCTONF,
                    NRSEQVENREC,
                    NRPARCELAREC
                 FROM CONTARECEBER
                 WHERE
                   (CDFILIBOLREC = C_BOLREC.CDFILIAL)  AND
                   (NRLANBOLREC  = C_BOLREC.NRLANBOLREC)) LOOP

                DELETE FROM RATCTRECEBER
                WHERE
                   (CDFILIAL     = C_CNTREC.CDFILIAL)     AND
                   (NRLANCTONF   = C_CNTREC.NRLANCTONF)   AND
                   (NRSEQVENREC  = C_CNTREC.NRSEQVENREC)  AND
                   (NRPARCELAREC = C_CNTREC.NRPARCELAREC);

                DELETE FROM AJCTRECEBER
                WHERE
                   (CDFILIAL     = C_CNTREC.CDFILIAL)     AND
                   (NRLANCTONF   = C_CNTREC.NRLANCTONF)   AND
                   (NRSEQVENREC  = C_CNTREC.NRSEQVENREC)  AND
                   (NRPARCELAREC = C_CNTREC.NRPARCELAREC);

                DELETE FROM CONTARECEBER
                WHERE
                   (CDFILIAL     = C_CNTREC.CDFILIAL)     AND
                   (NRLANCTONF   = C_CNTREC.NRLANCTONF)   AND
                   (NRSEQVENREC  = C_CNTREC.NRSEQVENREC)  AND
                   (NRPARCELAREC = C_CNTREC.NRPARCELAREC);
             END LOOP;

             DELETE FROM AJBOLREC
             WHERE
                (CDFILIAL    = C_BOLREC.CDFILIAL) AND
                (NRLANBOLREC = C_BOLREC.NRLANBOLREC);

             DELETE FROM ERRCNABBOLR
             WHERE
                (CDFILIAL    = C_BOLREC.CDFILIAL) AND
                (NRLANBOLREC = C_BOLREC.NRLANBOLREC);

             DELETE FROM BOLETRECEBER
             WHERE
                (CDFILIAL    = C_BOLREC.CDFILIAL) AND
                (NRLANBOLREC = C_BOLREC.NRLANBOLREC);
          END IF;
      END LOOP;
   END IF;

/*  EXCLUSO DAS BAIXAS PAGAS E DOS MOVIMENTOS  */

   SELECT
      COUNT(*)
   INTO
      V_CTBAIXA
   FROM BAIXAPAGA
   WHERE
      (CDEMPRESA  = P_CDEMPRESA) AND
      (DTBAIXAPAG <= P_DTLIMITE);

   IF V_CTBAIXA > 0 THEN
      FOR C_BAIXAPAG IN
         (SELECT
            CDFILIAL,
            NRBAIXAPAG,
            CDREDUCONT,
            NRMOVBANC
          FROM BAIXAPAGA
          WHERE
            (CDEMPRESA  = P_CDEMPRESA) AND
            (DTBAIXAPAG <= P_DTLIMITE)) LOOP

          SELECT
             COUNT(*)
          INTO
             V_CTBOL
          FROM BOLETPAGAR
          WHERE
             (CDFILIBAIPAG = C_BAIXAPAG.CDFILIAL) AND
             (NRBAIXAPAG   = C_BAIXAPAG.NRBAIXAPAG);

          IF (V_CTBOL = 0) THEN
             DELETE FROM BAIXAPAGA
             WHERE
                (CDFILIAL   = C_BAIXAPAG.CDFILIAL) AND
                (NRBAIXAPAG = C_BAIXAPAG.NRBAIXAPAG);

             DELETE FROM MOVBANCARIO
             WHERE
                (CDREDUCONT = C_BAIXAPAG.CDREDUCONT) AND
                (NRMOVBANC  = C_BAIXAPAG.NRMOVBANC);

             DELETE FROM MOVREAL
             WHERE
                (CDREDUCONT = C_BAIXAPAG.CDREDUCONT) AND
                (NRMOVBARE  = C_BAIXAPAG.NRMOVBANC);
          END IF;
      END LOOP;
   END IF;

/*  EXCLUSO DAS BAIXAS RECEBIDAS E DOS MOVIMENTOS  */

   SELECT
      COUNT(*)
   INTO
      V_CTBAIXA
   FROM BAIXARECEBIDA
   WHERE
      (CDEMPRESA  = P_CDEMPRESA) AND
      (DTBAIXAREC <= P_DTLIMITE);

   IF V_CTBAIXA > 0 THEN
      FOR C_BAIXAREC IN
         (SELECT
            CDFILIAL,
            NRBAIXAREC,
            CDREDUCONT,
            NRMOVBANC
          FROM BAIXARECEBIDA
          WHERE
            (CDEMPRESA  = P_CDEMPRESA) AND
            (DTBAIXAREC <= P_DTLIMITE)) LOOP

          SELECT
             COUNT(*)
          INTO
             V_CTBOL
          FROM BOLETRECEBER
          WHERE
             (CDFILIBAIREC = C_BAIXAREC.CDFILIAL) AND
             (NRBAIXAREC   = C_BAIXAREC.NRBAIXAREC);

          IF (V_CTBOL = 0) THEN
             DELETE FROM BAIXARECEBIDA
             WHERE
                (CDFILIAL   = C_BAIXAREC.CDFILIAL) AND
                (NRBAIXAREC = C_BAIXAREC.NRBAIXAREC);

             DELETE FROM MOVBANCARIO
             WHERE
                (CDREDUCONT = C_BAIXAREC.CDREDUCONT) AND
                (NRMOVBANC  = C_BAIXAREC.NRMOVBANC);

             DELETE FROM MOVREAL
             WHERE
                (CDREDUCONT = C_BAIXAREC.CDREDUCONT) AND
                (NRMOVBARE  = C_BAIXAREC.NRMOVBANC);
          END IF;
      END LOOP;
   END IF;

/*  EXCLUSO DE DEBITO E CREDITO  */

   SELECT
      COUNT(*)
   INTO
      V_CTDEBI
   FROM MOVDEBBANC
   WHERE
      (CDEMPRESA  = P_CDEMPRESA) AND
      (DTMOVIDEBI <= P_DTLIMITE);

   IF V_CTDEBI > 0 THEN
      FOR C_MOVDEBI IN
         (SELECT
             CDREDUCONT,
             NRMOVIDEBI,
             NRMOVBANC
          FROM MOVDEBBANC
          WHERE
            (CDEMPRESA  = P_CDEMPRESA) AND
            (DTMOVIDEBI <= P_DTLIMITE)) LOOP

            DELETE FROM MOVDEBBANC
            WHERE
               (CDREDUCONT = C_MOVDEBI.CDREDUCONT) AND
               (NRMOVIDEBI = C_MOVDEBI.NRMOVIDEBI);

            DELETE FROM MOVBANCARIO
            WHERE
               (CDREDUCONT = C_MOVDEBI.CDREDUCONT) AND
               (NRMOVBANC  = C_MOVDEBI.NRMOVBANC);

            DELETE FROM MOVREAL
            WHERE
               (CDREDUCONT = C_MOVDEBI.CDREDUCONT) AND
               (NRMOVBARE  = C_MOVDEBI.NRMOVBANC);
      END LOOP;
   END IF;


   SELECT
      COUNT(*)
   INTO
      V_CTCRED
   FROM MOVCREDBANC
   WHERE
      (CDEMPRESA  = P_CDEMPRESA) AND
      (DTMOVICRED <= P_DTLIMITE);

   IF V_CTCRED > 0 THEN
      FOR C_MOVCRED IN
         (SELECT
             CDREDUCONT,
             NRMOVICRED,
             NRMOVBANC
          FROM MOVCREDBANC
          WHERE
            (CDEMPRESA  = P_CDEMPRESA) AND
            (DTMOVICRED <= P_DTLIMITE)) LOOP

            DELETE FROM MOVCREDBANC
            WHERE
               (CDREDUCONT = C_MOVCRED.CDREDUCONT) AND
               (NRMOVICRED = C_MOVCRED.NRMOVICRED);

            DELETE FROM MOVBANCARIO
            WHERE
               (CDREDUCONT = C_MOVCRED.CDREDUCONT) AND
               (NRMOVBANC  = C_MOVCRED.NRMOVBANC);

            DELETE FROM MOVREAL
            WHERE
               (CDREDUCONT = C_MOVCRED.CDREDUCONT) AND
               (NRMOVBARE  = C_MOVCRED.NRMOVBANC);
      END LOOP;
   END IF;

/*  EXCLUSO DE TRANSFERENCIA  */

   SELECT
      COUNT(*)
   INTO
      V_CTTRANSF
   FROM TRANSFBANC
   WHERE
      (CDEMPRESA  = P_CDEMPRESA) AND
      (DTTRANBANC <= P_DTLIMITE);

   IF V_CTTRANSF > 0 THEN
      FOR C_TRANSF IN
         (SELECT
             TF.CDEMPRESA,
             TF.NRTRANBANC,
             IT.CDREDUCONT,
             IT.NRMOVBANC
          FROM
             ITTRANSFBANC IT,
             TRANSFBANC   TF
          WHERE
            (TF.CDEMPRESA  = P_CDEMPRESA)    AND
            (TF.DTTRANBANC <= P_DTLIMITE)    AND
            (IT.CDEMPRESA  = TF.CDEMPRESA)   AND
            (IT.NRTRANBANC = TF.NRTRANBANC)) LOOP

            DELETE FROM ITTRANSFBANC
            WHERE
               (CDEMPRESA  = C_TRANSF.CDEMPRESA) AND
               (NRTRANBANC = C_TRANSF.NRTRANBANC);

            DELETE FROM MOVBANCARIO
            WHERE
               (CDREDUCONT  = C_TRANSF.CDREDUCONT) AND
               (NRMOVBANC   = C_TRANSF.NRMOVBANC);

            DELETE FROM MOVREAL
            WHERE
               (CDREDUCONT  = C_TRANSF.CDREDUCONT) AND
               (NRMOVBARE   = C_TRANSF.NRMOVBANC);
      END LOOP;

      DELETE FROM TRANSFBANC
      WHERE
         (CDEMPRESA  = P_CDEMPRESA) AND
         (DTTRANBANC <= P_DTLIMITE);
   END IF;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('LIMPA_FINANCEIRO','4.09.066'); END;
/

