function crm00042000_concorrente_controller(General, ScreenService, Endereco) {
    
    
    /**
     * Seta as váriaveis de paramsNewCode 
     * para um novo código ser gerado.
     * 
     * @param {object} widget widget de concorrente
     * para setar os parametros de novo código
    */
    this.enableNewCodeNumber = function(widget){
        var paramsNewCode = widget.paramsNewCode;
        
        paramsNewCode.__is_new_code = true;
        paramsNewCode.__new_code_pk = "NRCONCORRENTE";
        paramsNewCode.__table_pk = "NRCONCORRENTE";
        paramsNewCode.__cast_number = true;
    };
    
    
    /**
     * Seta as váriaveis de paramsNewCode 
     * para um novo código não ser gerado.
     * 
     * @param {object} widget widget de concorrente
     * para setar os parametros de novo código
     */
    this.desableNewCodeNumber = function(widget){
        var paramsNewCode = widget.paramsNewCode;
        
        paramsNewCode.__is_new_code = false;
        paramsNewCode.__new_code_pk = "";
        paramsNewCode.__table_pk = "";
        paramsNewCode.__cast_number = true;
    };
    
    /**
     * Remove tudo que não for numerico
     * da inscrição.
     * 
     * @param {varchar} inscription código de inscrição
     * digitado pelo usuário.
     */
    this.removeInscriptionMask = function(inscription){
        return inscription.replace(/[^0-9]/g, "");
    };
    
    
    /**
     * Salva o registro de concorrente
     * 
     * @param {object} widget widget de concorrente
     * para setar campos da mesma
     */
    this.saveConcorrente = function(widget){
        
        this.removeInscriptionMask(widget.currentRow.NRINSCRICAOCONC);
        
        if(!widget.currentRow.NMFANTCONN){
            widget.currentRow.NMFANTCONN = widget.currentRow.NMRAZSOCCONC;
        }
        
        if(widget.currentRow.__is_new){
            this.enableNewCodeNumber(widget);
        }
        
        widget.currentRow.__columns_to_return = ["NRCONCORRENTE"];
        
        General.widgetSave(widget).then(function(response){
            if (response.dataset.columnsToReturn)
                widget.currentRow.NRCONCORRENTE = response.dataset.columnsToReturn.NRCONCORRENTE;
                widget.reload();
            this.desableNewCodeNumber(widget);
        }.bind(this));
    };
    
    /**
     * Seta os campos para que não sejam nulos 
     * e possam ser salvos sem serem preenchidos 
     * pelo usuário
     * 
     * @param {object} currentRow setar os campos
     * da currentRow com "" para evitar erro quando
     * as variáveis são nulas.
     */
    this.setDefaultValues = function(currentRow){
        currentRow.CDPAIS = "";
        currentRow.CDMUNICIPIO = "";
        currentRow.SGESTADO = "";
        currentRow.CDBAIRRO = "";
    };
    
      /**
     * Valida a inscrição informada pelo usuário
     * e seta na currentRow da widget o código do
     * tipo da inscrição através do método 
     * 'setInscriptionTypeInCurrentRow', e a máscara
     * da inscrição, caso a inscrição seja inválida,
     * ele limpa os campos e manda uma mensagem.
     * 
     * @param {object} widget widget onde está sendo
     * realizado o cadastro da revenda.
    */
    this.validateInscription = function(widget){
        
        var currentRow = widget.currentRow;
        
        var resultValidate = General.inscricaoFormat(currentRow.NRINSCRICAOCONC);
        
        if(resultValidate.valid){
            
            this.setInscriptionTypeInCurrentRow(widget, resultValidate);
            
            currentRow.NRINSCRICAOCONC = resultValidate.value;
            
        }else{
            
            this.showErrorMessageForInscription(currentRow.NRINSCRICAOCONC.length);
            
            currentRow.NRINSCRICAOCONC = "";
            currentRow.CDTIPOINSCRICAO = "";
        }
        
    };
    
    /**
     * Caso o usuário digite
     * uma inscrição inválida
     * mostra os erros correspondentes.
     * 
     * @param {varchar} inscription mostrar a mensagem
     * de erro dependendo do tipo do mesmo quando a 
     * inscrição é inválida.
     */
    this.showErrorMessageForInscription = function(inscription){
        switch(inscription){
            
            case 11:
                ScreenService.showMessage("Número de CPF inválido.");
                break;
            case 14:
                ScreenService.showMessage("Número de CNPJ inválido.");
                break;
            default:
                ScreenService.showMessage("Número de incrição inválido.");
        }
        
    };
    
    /**
     * Seta na currentRow da widget, o nome
     * e o código da inscrição informada.
     * 
     * @param {object} widget onde está sendo 
     * realizada o cadastro da revena.
     * @param {object} resultValidade reultado 
     * da validação da inscrição.
    */
    this.setInscriptionTypeInCurrentRow = function(widget, resultValidate){
        
        var inscriptionCod = resultValidate.tipo.cod;
        var currentRow = widget.currentRow;
        
        if (inscriptionCod){
            
            switch(inscriptionCod){
                
                case "F":
                    currentRow.CDTIPOINSCRICAO = "CPF";
                break;
                
                case "J":
                    currentRow.CDTIPOINSCRICAO = "CNPJ";
                break;
                
                case "O":
                    currentRow.CDTIPOINSCRICAO = "CODIGO";
                break;
                
                case "I":
                    currentRow.CDTIPOINSCRICAO = "LIVRE";
                break;
            }
            
        }
    };
    
    /**
     * Monta o filtro para os campos Estado, Pais e Municipio atraveś de um switch
     * 
     * @param {object} widget widget de concorrente
     * para pegar a currentRow
     * @param {object} field campo utilizado para pegar 
     * o nome e dataSourceFilter do campo.
    */
    this.filterAddress = function(widget, field){
        var currentRow = widget.currentRow;
        var filter = [];
        
        switch (field.name) {
            case 'SGESTADO':
                if (currentRow.CDPAIS){
                   filter = this.createFilterAddress("CDPAIS",currentRow.CDPAIS); 
                }else{
                     filter = this.createFilterAddress("CDPAIS","");  
                }
                break;
            case 'CDMUNICIPIO':
                if (currentRow.SGESTADO){
                    filter = this.createFilterAddress("SGESTADO",currentRow.SGESTADO);
                }else{
                     filter = this.createFilterAddress("SGESTADO","");  
                }
                break;
            case 'NMBAIRRO':
                if (currentRow.CDMUNICIPIO){
                    filter = this.createFilterAddress("CDMUNICIPIO",currentRow.CDMUNICIPIO);
                }else{
                     filter = this.createFilterAddress("CDMUNICIPIO","");  
                }
                break;
        }
        var nrorgfilter = {};
        nrorgfilter.name = "NRORG";
        nrorgfilter.operator = "IN";
        nrorgfilter.value = [getLocalVar('CRM_NRORGTRAB'),getLocalVar('CRM_NRORGPADRAO') ];
        
        filter.push(nrorgfilter);
        field.dataSourceFilter = filter;
        field.reload();
    };
    
    
    /**
     * Monta o filtro para os campos Estado, Pais e Municipio 
     * pegando os parametros do método filterAddress
     * 
     * @param {string} name nome do campo 
     * de estado país ou municípo
     * @param {varchar} value valor utilizado para setar
     * o valor do filtro.
    */
    this.createFilterAddress = function(name, value){
        var filter = [
            {
                "name": name,
                "operator": "=",
                "value": value
            }
        ];
        return filter;
    };
    
    var oldState;
    
    /**
     * Limpa os campos de municipio e bairro quando o 
     * campo Estado é alterado.
     * 
     * @param {object} currentRow pega a currentRow
     * para setar os campos com vazio.
    */
    
    this.onChangeState = function(currentRow){
      var currentState = currentRow.NMESTADO.toString();

      if(typeof oldState === "undefined"){
          oldState = currentState;
          
      }else if(oldState !== currentState){
          currentRow.NMMUNICIPIO = "";
          currentRow.CDMUNICIPIO = "";
          currentRow.NMBAIRRO = "";
          currentRow.CDBAIRRO = "";
          oldState = currentState;
      }
     };
     
    var oldCountry;
    /**
     * Limpa os campos de estado, municipio e bairro quando o 
     * campo País é alterado.
     * 
     * @param {object} currentRow pega a currentRow
     * para setar os campos com vazio.
    */
    this.onChangeCountry = function(currentRow){
      var currentCountry = currentRow.NMPAIS.toString();

      if(typeof oldCountry === "undefined"){
          oldCountry = currentCountry;
          
      }else if(oldCountry !== currentCountry){
          currentRow.SGESTADO = "";
          currentRow.NMMUNICIPIO = "";
          currentRow.CDMUNICIPIO = "";
          currentRow.NMBAIRRO = "";
          currentRow.CDBAIRRO = "";
          oldCountry = currentCountry;
        }
    };
    
    var oldCity;
    /**
     * Limpa os campos de bairro quando o 
     * campo Município é alterado.
     * 
     * @param {object} currentRow pega a currentRow
     * para setar os campos com vazio.
    */
    this.onChangeCity = function(currentRow){
      var currentCity = currentRow.NMMUNICIPIO.toString();

      if(typeof oldCity === "undefined"){
          oldCity = currentCity;
          
      }else if(oldCity !== currentCity){
          currentRow.NMBAIRRO = "";
          currentRow.CDBAIRRO = "";
          oldCity = currentCity;
        }
    };
    
    /**
     * Busca os campos de País, Estado, Município e Bairro
     * a partir do CEP informado pelo usuário.
     * 
     * @param {object} currentRow pega a currentRow
     * para setar os campos com os valores buscados
     * @param {object} CEP valor digitado pelo usuário 
     * do CEP a ser buscado
    */
    this.searchAddress = function(currentRow, CEP){
        
        ScreenService.confirmMessage("Deseja buscar o endereço pelo CEP informado?", "question", function(){
            Endereco.buscarCep(CEP).then(function(response) {
                var endereco = response.dataset.ENDERECO;
                if(endereco) {
                    currentRow.CEP = CEP;
                    currentRow.CDPAIS = (endereco.CDPAIS)? endereco.CDPAIS : '';
                    currentRow.NMPAIS = (endereco.NMPAIS)? endereco.NMPAIS : '';
                    currentRow.SGESTADO = (endereco.SGESTADO)? endereco.SGESTADO : '';
                    currentRow.NMESTADO = (endereco.NMESTADO)? endereco.NMESTADO : '';
                    currentRow.CDMUNICIPIO = (endereco.CDMUNICIPIO)? endereco.CDMUNICIPIO : '';
                    currentRow.NMMUNICIPIO = (endereco.NMMUNICIPIO)? endereco.NMMUNICIPIO : '';
                    currentRow.CDBAIRRO = (endereco.CDBAIRRO)? endereco.CDBAIRRO : '';
                    currentRow.NMBAIRRO = (endereco.NMBAIRRO)? endereco.NMBAIRRO : '';
                    currentRow.DSENDERECOCONC = (endereco.DSENDERECO)? endereco.DSENDERECO : '';
                    
                    /*TODO Encontrar um jeito melhor de fazer o autocomplete atualizar os fields para os dados da currentRow*/
                    angular.element('#footer').click();
                    
                } else {
                    currentRow.CEP = '';
                }
                if(response.dataset.ERRO) ScreenService.showMessage(response.dataset.ERRO.MESSAGE, "warning");
            }).catch(function(error){
                ScreenService.showMessage('Erro ao realizar a busca.');
            });
        }, null);
        
        ScreenService.closePopup();
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('crm00042000_concorrente_controller', crm00042000_concorrente_controller);
});